@@ -97,13 +97,27 @@ class FloatItem(NumericTypeItem):
         * default [float]: default value (optional)
         * min [float]: minimum value (optional)
         * max [float]: maximum value (optional)
+        * slider [bool]: if True, shows a slider widget right after the line 
+          edit widget (default is False)
+        * step [float]: step between tick values with a slider widget (optional)
         * nonzero [bool]: if True, zero is not a valid value (optional)
         * unit [string]: physical unit (optional)
         * help [string]: text shown in tooltip (optional)
     """
     type = float
-    
-    
+    def __init__(self, label, default=None, min=None, max=None,
+                 nonzero=None, unit='', step=0.1, slider=False, help=''):
+        super(FloatItem, self).__init__(label, default=default, min=min,
+                                max=max, nonzero=nonzero, unit=unit, help=help)
+        self.set_prop("display", slider=slider)
+        self.set_prop("data", step=step)
+
+    def get_value_from_reader(self, reader):
+        """Reads value from the reader object, inside the try...except 
+        statement defined in the base item `deserialize` method"""
+        return reader.read_float()
+
+
 class IntItem(NumericTypeItem):
     """
     Construct an integer data item
@@ -150,6 +164,11 @@ class IntItem(NumericTypeItem):
                 return False
         return True
 
+    def get_value_from_reader(self, reader):
+        """Reads value from the reader object, inside the try...except 
+        statement defined in the base item `deserialize` method"""
+        return reader.read_int()
+
 
 class StringItem(DataItem):
     """
@@ -157,13 +176,15 @@ class StringItem(DataItem):
         * label [string]: name
         * default [string]: default value (optional)
         * help [string]: text shown in tooltip (optional)
-    notempty [bool]: if True, empty string is not a valid value (optional)
+        * notempty [bool]: if True, empty string is not a valid value (opt.)
+        * wordwrap [bool]: toggle word wrapping (optional)
     """
     type = (unicode, str)
-    def __init__(self, label, default=None,
-                 notempty=None, help=''):
+    def __init__(self, label, default=None, notempty=None,
+                 wordwrap=False, help=''):
         DataItem.__init__(self, label, default=default, help=help)
         self.set_prop("data", notempty=notempty)
+        self.set_prop("display", wordwrap=wordwrap)
 
     def check_value(self, value):
         """Override DataItem method"""
@@ -177,6 +198,11 @@ class StringItem(DataItem):
         # QString -> str
         return unicode(value)
 
+    def get_value_from_reader(self, reader):
+        """Reads value from the reader object, inside the try...except 
+        statement defined in the base item `deserialize` method"""
+        return reader.read_unicode()
+
 
 class TextItem(StringItem):
     """
@@ -184,8 +210,13 @@ class TextItem(StringItem):
         * label [string]: name
         * default [string]: default value (optional)
         * help [string]: text shown in tooltip (optional)
+        * notempty [bool]: if True, empty string is not a valid value (opt.)
+        * wordwrap [bool]: toggle word wrapping (optional)
     """
-    pass
+    def __init__(self, label, default=None, notempty=None,
+                 wordwrap=True,help=''):
+        StringItem.__init__(self, label, default=default, notempty=notempty,
+                            wordwrap=wordwrap, help=help)
 
 
 class BoolItem(DataItem):
@@ -201,6 +232,11 @@ class BoolItem(DataItem):
         DataItem.__init__(self, label, default=default, help=help)
         self.set_prop("display", text=text)
 
+    def get_value_from_reader(self, reader):
+        """Reads value from the reader object, inside the try...except 
+        statement defined in the base item `deserialize` method"""
+        return reader.read_bool()
+
 
 class DateItem(DataItem):
     """
@@ -232,12 +268,20 @@ class ColorItem(StringItem):
         from guidata.qthelpers import text_to_qcolor
         return text_to_qcolor(value).isValid()
 
+    def get_value_from_reader(self, reader):
+        """Reads value from the reader object, inside the try...except 
+        statement defined in the base item `deserialize` method"""
+        # Using read_str converts `numpy.string_` to `str` -- otherwise,
+        # when passing the string to a QColor Qt object, any numpy.string_ will
+        # be interpreted as no color (black)
+        return reader.read_str()
+
 
 class FileSaveItem(StringItem):
     """
     Construct a path data item for a file to be saved
         * label [string]: name
-        * formats [string (or string list)]: wildcard filter (e.g. '*.txt')
+        * formats [string (or string list)]: wildcard filter
         * default [string]: default value (optional)
         * basedir [string]: default base directory (optional)
         * help [string]: text shown in tooltip (optional)
@@ -272,7 +316,7 @@ class FileOpenItem(FileSaveItem):
     """
     Construct a path data item for a file to be opened
         * label [string]: name
-        * formats [string (or string list)]: wildcard filter (e.g. '*.txt')
+        * formats [string (or string list)]: wildcard filter
         * default [string]: default value (optional)
         * basedir [string]: default base directory (optional)
         * help [string]: text shown in tooltip (optional)
@@ -288,7 +332,7 @@ class FilesOpenItem(FileSaveItem):
     """
     Construct a path data item for multiple files to be opened.
         * label [string]: name
-        * formats [string (or string list)]: wildcard filter (e.g. '*.txt')
+        * formats [string (or string list)]: wildcard filter
         * default [string]: default value (optional)
         * basedir [string]: default base directory (optional)
         * help [string]: text shown in tooltip (optional)
@@ -319,6 +363,16 @@ class FilesOpenItem(FileSaveItem):
             value = [value]
         return [add_extension(self, path) for path in value]
 
+    def serialize(self, instance, writer):
+        """Serialize this item"""
+        value = self.get_value(instance)
+        writer.write_sequence([fname.encode("utf-8") for fname in value])
+
+    def get_value_from_reader(self, reader):
+        """Reads value from the reader object, inside the try...except 
+        statement defined in the base item `deserialize` method"""
+        return [unicode(fname, "utf-8") for fname in reader.read_sequence()]
+
 
 class DirectoryItem(StringItem):
     """
@@ -393,8 +447,7 @@ class ChoiceItem(DataItem):
         
 class MultipleChoiceItem(ChoiceItem):
     """
-    Construct a data item for a list of choices -- multiple choices can be 
-    selected
+    Construct a data item for a list of choices -- multiple choices can be selected
         * label [string]: name
         * choices [list or tuple]: string list or (key, label) list
         * default [-]: default label or default key (optional)
@@ -425,8 +478,7 @@ class MultipleChoiceItem(ChoiceItem):
         return self
 
     def serialize(self, instance, writer):
-        """Deserialize this item 
-        """
+        """Serialize this item"""
         value = self.get_value(instance)
         seq = []
         _choices = self.get_prop_value("data", instance, "choices")
@@ -435,8 +487,7 @@ class MultipleChoiceItem(ChoiceItem):
         writer.write_sequence( seq )
 
     def deserialize(self, instance, reader):
-        """Deserialize this item 
-        """
+        """Deserialize this item"""
         flags = reader.read_sequence()
         # We could have trouble with objects providing their own choice
         # function which depend on not yet deserialized values
@@ -481,6 +532,7 @@ class FloatArrayItem(DataItem):
         * help [string]: text shown in tooltip (optional)
         * format [string]: formatting string (example: '%.3f') (optional)
         * transpose [bool]: transpose matrix (display only)
+        * large [bool]: view all float of the array
         * minmax [string]: "all" (default), "columns", "rows"
     """
     def __init__(self, label, default=None, help='', format='%.3f',
@@ -491,8 +543,31 @@ class FloatArrayItem(DataItem):
         
     def format_string(self, instance, value, fmt, func):
         """Override DataItem method"""
-        v = func(value)
-        return u"~= %g [%g .. %g]" % (v.mean(), v.min(), v.max())
+        larg = self.get_prop_value("display", instance, "large", False)
+        fmt = self.get_prop_value("display", instance, "format", u"%s")
+        v = func(value)        
+        if larg:
+            text = u"= ["
+            for flt in v[:-1]:
+                text += fmt % flt + "; "
+            text += fmt % v[-1] + "]"
+            return unicode(text)
+        else:
+            text = u"~= " + fmt % v.mean()
+            text += " [" + fmt % v.min()
+            text += " .. " + fmt % v.max()
+            text += "]"
+            return unicode(text)
+
+    def serialize(self, instance, writer):
+        """Serialize this item"""
+        value = self.get_value(instance)
+        writer.write_array(value)
+
+    def get_value_from_reader(self, reader):
+        """Reads value from the reader object, inside the try...except 
+        statement defined in the base item `deserialize` method"""
+        return reader.read_array()
 
 
 class ButtonItem(DataItem):
