@@ -17,55 +17,14 @@ ValueProp, ...).
 # pylint: disable-msg=W0622
 # pylint: disable-msg=W0212
 
-import sys, re
+import sys
+import re
 
 from guidata.utils import utf8_to_unicode, update_dataset
 
 
 DEBUG_DESERIALIZE = False
 
-
-class IDataItemWriter(object):
-    """An interface for dataitem/dataset serialization"""
-    def write_int(self, ival):
-        pass
-    def write_float(self, fval):
-        pass
-    def write_unicode(self, uval):
-        pass
-    def write_array(self, aval):
-        pass
-    def write_sequence(self, seq):
-        pass
-    def write_any(self, any):
-        """try to write any python object,
-        could resort to using pickle"""
-        pass
-    def write_none(self):
-        pass
-    def begin(self, section):
-        pass
-    def end(self, section):
-        pass
-
-class IDataItemReader(object):
-    """An interface for dataitem/dataset serialization"""
-    def read_int(self):
-        pass
-    def read_float(self):
-        pass
-    def read_unicode(self):
-        pass
-    def read_array(self):
-        pass
-    def read_sequence(self):
-        pass
-    def read_any(self):
-        pass
-    def begin(self, section):
-        pass
-    def end(self, section):
-        pass
  
 class NoDefault:
     pass
@@ -152,7 +111,7 @@ class NotProp(ItemProperty):
     def set(self, instance, item, value):
         self.property.set(instance, item, not value)
 
-class OperatorProperty(ItemProperty):
+class FuncProp(ItemProperty):
     """An 'operator property'
     prop: ItemProperty instance
     func: function
@@ -178,8 +137,7 @@ class DataItem(object):
     
     `label` : (str, unicode)
     `default` : any type, optional
-    `help` : (str, unicode)
-        Text displayed on data item's tooltip
+    `help` : (str, unicode) Text displayed on data item's tooltip
     """
     count = 0
 
@@ -360,29 +318,15 @@ class DataItem(object):
         this is a default implementation that should work for
         everything but new datatypes
         """
-        from numpy import ndarray
         value = self.get_value(instance)
-        if isinstance(value, bool):
-            writer.write_bool(value)
-        elif isinstance(value, int):
-            writer.write_int(value)
-        elif isinstance(value, float):
-            writer.write_int(value)
-        elif isinstance(value, unicode):
-            writer.write_unicode(value)
-        elif isinstance(value, str):
-            writer.write_any(value)
-        elif isinstance(value, ndarray):
-            writer.write_array(value)
-        elif isinstance(value, DataSet):
-            value.serialize(writer)
-        elif value is None:
-            writer.write_none()
-        elif isinstance(value, (list, tuple)):
-            writer.write_sequence(value)
-        else:
-            raise NotImplementedError("cannot serialize %r of type %r" %
-                                      (value, type(value)))
+        writer.write(value)
+
+    def get_value_from_reader(self, reader):
+        """Reads value from the reader object, inside the try...except 
+        statement defined in the base item `deserialize` method
+        
+        This method is reimplemented in some child classes"""
+        return reader.read_any()
 
     def deserialize(self, instance, reader):
         """Deserialize this item using the reader object
@@ -391,16 +335,16 @@ class DataItem(object):
         detect expected datatype from the stream
         """
         try:
-            value = reader.read_any()
+            value = self.get_value_from_reader(reader)
         except RuntimeError, e:
             if DEBUG_DESERIALIZE:
                 import traceback
                 print >>sys.stderr, "DEBUG_DESERIALIZE enabled in datatypes.py"
                 traceback.print_stack()
                 print >>sys.stderr, e
-            self.set_default( instance )
+            self.set_default(instance)
             return
-        self.__set__( instance, value)
+        self.__set__(instance, value)
         
 
 class Obj(object):
@@ -717,21 +661,27 @@ class DataSet(object):
         import textedit
         self.accept(textedit.TextEditVisitor(self))
 
-    def edit(self, parent=None, apply=None):
+    def edit(self, parent=None, apply=None, size=None):
         """
         Open a dialog box to edit data set
+            * parent: parent widget (default is None, meaning no parent)
+            * apply: apply callback (default is None)
+            * size: dialog size (QSize object or integer tuple (width, height))
         """
         from guidata.dataset.qtwidgets import DataSetEditDialog
         win = DataSetEditDialog(self, icon=self.__icon, parent=parent,
-                                apply=apply)
+                                apply=apply, size=size)
         return win.exec_()
     
-    def view(self, parent=None):
+    def view(self, parent=None, size=None):
         """
         Open a dialog box to view data set
+            * parent: parent widget (default is None, meaning no parent)
+            * size: dialog size (QSize object or integer tuple (width, height))
         """
         from guidata.dataset.qtwidgets import DataSetShowDialog
-        win = DataSetShowDialog(self, icon=self.__icon, parent=parent)
+        win = DataSetShowDialog(self, icon=self.__icon, parent=parent,
+                                size=size)
         return win.exec_()
         
     def to_string(self, debug=False, indent=None, align=False):
@@ -792,23 +742,30 @@ class DataSet(object):
 
     def serialize(self, writer):
         for item in self._items:
-            writer.begin(item._name)
-            item.serialize(self, writer)
-            writer.end(item._name)
+            with writer.group(item._name):
+                item.serialize(self, writer)
 
     def deserialize(self, reader):
         for item in self._items:
-            reader.begin(item._name)
-            item.deserialize(self, reader)
-            reader.end(item._name)
+            with reader.group(item._name):
+                try:
+                    item.deserialize(self, reader)
+                except RuntimeError, error:
+                    if DEBUG_DESERIALIZE:
+                        import traceback
+                        print >>sys.stderr,\
+                            "DEBUG_DESERIALIZE enabled in datatypes.py"
+                        traceback.print_stack()
+                        print >>sys.stderr, error
+                    item.set_default(self)
             
     def read_config(self, conf, section, option):
-        from guidata.userconfig import UserConfigReader 
+        from guidata.userconfigio import UserConfigReader 
         reader = UserConfigReader(conf, section, option)
         self.deserialize(reader)
         
     def write_config(self, conf, section, option):
-        from guidata.userconfig import UserConfigWriter 
+        from guidata.userconfigio import UserConfigWriter 
         writer = UserConfigWriter(conf, section, option)
         self.serialize(writer)
 
