@@ -14,7 +14,9 @@ There is one widget type for each data item type.
 Example: ChoiceWidget <--> ChoiceItem, ImageChoiceItem
 """
 
-import os, sys, numpy
+import os
+import sys
+import numpy
 
 try:
     # PyQt4 4.3.3 on Windows (static DLLs) with py2exe installed:
@@ -29,7 +31,7 @@ from guidata.qt.QtGui import (QIcon, QPixmap, QHBoxLayout, QGridLayout,
                               QColorDialog, QPushButton, QLineEdit, QCheckBox,
                               QComboBox, QWidget, QTabWidget, QGroupBox,
                               QLabel, QTextEdit, QFrame, QDateEdit, QSlider,
-                              QDateTimeEdit)
+                              QDateTimeEdit, QColor)
 from guidata.qt.QtCore import Qt, QObject, SIGNAL
 from guidata.qt.compat import getexistingdirectory
 try:
@@ -71,6 +73,7 @@ class AbstractDataSetWidget(object):
         self.parent_layout = parent_layout
         self.group = None # Layout/Widget grouping items
         self.label = None
+        self.build_mode = False
 
     def place_label(self, layout, row, column):
         """
@@ -239,10 +242,11 @@ class LineEditWidget(AbstractDataSetWidget):
         value = self.item.get()
         old_value = unicode(self.value())
         if value is not None:
+            if isinstance(value, QColor):  # if item is a ColorItem object
+                value = value.name()
             uvalue = utf8_to_unicode(value)
-            if uvalue!=old_value:
+            if uvalue != old_value:
                 self.edit.setText(utf8_to_unicode(value))
-                self.line_edit_changed(value)
         else:
             self.line_edit_changed(value)
             
@@ -255,7 +259,12 @@ class LineEditWidget(AbstractDataSetWidget):
             self.edit.setStyleSheet("")
             cb = self.item.get_prop_value("display", "callback", None)
             if cb is not None:
+                if self.build_mode:
+                    self.set()
+                else:
+                    self.parent_layout.update_dataitems()
                 cb(self.item.instance, self.item.item, value)
+                self.parent_layout.update_widgets(except_this_one=self)
         self.update(value)
         
     def update(self, value):
@@ -393,6 +402,7 @@ class DateTimeWidget(AbstractDataSetWidget):
     def __init__(self, item, parent_layout):
         super(DateTimeWidget, self).__init__(item, parent_layout)
         self.dateedit = self.group = QDateTimeEdit()
+        self.dateedit.setCalendarPopup(True)
         self.dateedit.setToolTip(item.get_help())
         
     def get(self):
@@ -444,7 +454,7 @@ class ColorWidget(HLayoutMixin, LineEditWidget):
         super(ColorWidget, self).__init__(item, parent_layout)
         self.button = QPushButton("")
         self.button.setMaximumWidth(32)
-        QObject.connect(self.button, SIGNAL("clicked()"), self.select_color )
+        QObject.connect(self.button, SIGNAL("clicked()"), self.select_color)
         self.group.addWidget(self.button)
         
     def update(self, value):
@@ -473,7 +483,7 @@ class ColorWidget(HLayoutMixin, LineEditWidget):
 
 class SliderWidget(HLayoutMixin, LineEditWidget):
     """
-    IntItem with Slider
+    NumericTypeItem with Slider
     """
     def __init__(self, item, parent_layout):
         super(SliderWidget, self).__init__(item, parent_layout)
@@ -482,8 +492,12 @@ class SliderWidget(HLayoutMixin, LineEditWidget):
             self.slider.setOrientation(Qt.Horizontal)
             vmin = item.get_prop_value("data", "min")
             vmax = item.get_prop_value("data", "max")
-            assert vmin is not None and vmax is not None, "SliderWidget requires that IntItem min/max have been defined"
-            self.slider.setRange(vmin, vmax)
+            vstep = item.get_prop_value("data", "step", 1)
+            assert vmin is not None and vmax is not None, "SliderWidget requires that NumericTypeItem min/max have been defined"
+            
+            self.slider.setRange(0, int((vmax-vmin)/vstep))
+            self.convert_value = lambda v: v*vstep + vmin
+            
             QObject.connect(self.slider, SIGNAL("valueChanged(int)"),
                             self.value_changed)
             self.group.addWidget(self.slider)
@@ -500,7 +514,7 @@ class SliderWidget(HLayoutMixin, LineEditWidget):
             
     def value_changed(self, ivalue):
         """Update the lineedit"""
-        value = str(ivalue)
+        value = str(self.convert_value(ivalue))
         self.edit.setText(value)
         self.update(value)
 
@@ -594,8 +608,27 @@ class ChoiceWidget(AbstractDataSetWidget):
         super(ChoiceWidget, self).__init__(item, parent_layout)
         self.combobox = self.group = QComboBox()
         self.combobox.setToolTip(item.get_help())
+        
+        self.__first_call = True
+        self.store = self.item.get_prop("display", "store", None)
+        QWidget.connect(self.combobox, SIGNAL("currentIndexChanged(int)"),
+                        self.index_changed)
+        
+    def index_changed(self, index):
+        if self.store:
+            self.store.set(self.item.instance, self.item.item, self.value())
+            self.parent_layout.refresh_widgets()
+        cb = self.item.get_prop_value("display", "callback", None)
+        if cb is not None:
+            if self.build_mode:
+                self.set()
+            else:
+                self.parent_layout.update_dataitems()
+            cb(self.item.instance, self.item.item, self.value())
+            self.parent_layout.update_widgets(except_this_one=self)
     
     def fill_combo(self):
+        self.combobox.blockSignals(True)
         while self.combobox.count():
             self.combobox.removeItem(0)
         _choices = self.item.get_prop_value("data", "choices")
@@ -610,6 +643,7 @@ class ChoiceWidget(AbstractDataSetWidget):
                 self.combobox.addItem(img, lbl)
             else:
                 self.combobox.addItem(lbl)
+        self.combobox.blockSignals(False)
         
     def get(self):
         """Override AbstractDataSetWidget method"""
@@ -619,20 +653,28 @@ class ChoiceWidget(AbstractDataSetWidget):
             idx = 0
             _choices = self.item.get_prop_value("data", "choices")
             for key, _val, _img in _choices:
-                if key==value:
+                if key == value:
                     break
-                idx = idx+1
+                idx += 1
+            self.combobox.blockSignals(True)
             self.combobox.setCurrentIndex(idx)
+            self.combobox.blockSignals(False)
+            if self.__first_call:
+                self.index_changed(idx)
+                self.__first_call = False
         
     def set(self):
         """Override AbstractDataSetWidget method"""
-        index = self.value()
-        choices = self.item.get_prop_value("data", "choices")
-        if index >= 0 and index < len(choices):
-            self.item.set(choices[index][0])
+        try:
+            value = self.value()
+        except IndexError:
+            return
+        self.item.set(value)
 
     def value(self):
-        return self.combobox.currentIndex()
+        index = self.combobox.currentIndex()
+        choices = self.item.get_prop_value("data", "choices")
+        return choices[index][0]
         
 class MultipleChoiceWidget(AbstractDataSetWidget):
     """
@@ -814,20 +856,12 @@ class ButtonWidget(AbstractDataSetWidget):
         layout.addWidget(self.group, row, label_column, row_span, column_span+1)
 
     def clicked(self, *args):
-        for widget in self.parent_layout.widgets:
-            # widget may have been modified, so we update the dataset
-            widget.set()
+        self.parent_layout.update_dataitems()
         callback = self.item.get_prop_value("display", "callback")
-        inst = self.item.instance
-        item = self.item.item
-        value = self.cb_value
-        parent = self.button.parent()
-        self.cb_value = callback(inst, item, value, parent)
+        self.cb_value = callback(self.item.instance, self.item.item,
+                                 self.cb_value, self.button.parent())
         self.set()
-        for widget in self.parent_layout.widgets:
-            # instance may have been modified so we update all the
-            # widgets of the instance
-            widget.get()
+        self.parent_layout.update_widgets()
 
 
 class DataSetWidget(AbstractDataSetWidget):
