@@ -30,7 +30,7 @@ from guidata.qt.QtGui import (QDialog, QMessageBox, QDialogButtonBox, QWidget,
                               QVBoxLayout, QGridLayout, QLabel, QSpacerItem,
                               QColor, QTabWidget, QIcon, QApplication, QPainter,
                               QPicture, QBrush, QGroupBox, QPushButton)
-from guidata.qt.QtCore import SIGNAL, SLOT, Qt, QRect
+from guidata.qt.QtCore import SIGNAL, SLOT, Qt, QRect, QSize
 from guidata.qt.compat import getopenfilename, getopenfilenames, getsavefilename
 
 from guidata.configtools import get_icon
@@ -44,13 +44,15 @@ class DataSetEditDialog(QDialog):
     """
     Dialog box for DataSet editing
     """
-    def __init__(self, instance, icon='', parent=None, apply=None):
+    def __init__(self, instance, icon='', parent=None, apply=None,
+                 wordwrap=True, size=None):
         QDialog.__init__(self, parent)
+        self.wordwrap = wordwrap
         self.apply_func = apply
         self.layout = QVBoxLayout()
         if instance.get_comment():
             label = QLabel(instance.get_comment())
-            label.setWordWrap(True)
+            label.setWordWrap(wordwrap)
             self.layout.addWidget(label)
         self.instance = instance
         self.edit_layout = [  ]
@@ -73,11 +75,17 @@ class DataSetEditDialog(QDialog):
         
         if parent is None:
             if not isinstance(icon, QIcon):
-                icon = get_icon(icon, default="guidata.png")
+                icon = get_icon(icon, default="guidata.svg")
             self.setWindowIcon(icon)
         
         self.setModal(True)
         self.setWindowTitle(instance.get_title())
+        
+        if size is not None:
+            if isinstance(size, QSize):
+                self.resize(size)
+            else:
+                self.resize(*size)
 
     def button_clicked(self, button):
         role = self.bbox.buttonRole(button)
@@ -144,7 +152,7 @@ class DataSetGroupEditDialog(DataSetEditDialog):
             layout.setAlignment(Qt.AlignTop)
             if dataset.get_comment():
                 label = QLabel(dataset.get_comment())
-                label.setWordWrap(True)
+                label.setWordWrap(self.wordwrap)
                 layout.addWidget(label)
             grid = QGridLayout()
             self.edit_layout.append( self.layout_factory(dataset, grid) )
@@ -305,6 +313,7 @@ DataSetEditLayout.register(FloatItem, LineEditWidget)
 DataSetEditLayout.register(StringItem, LineEditWidget)
 DataSetEditLayout.register(TextItem, TextEditWidget)
 DataSetEditLayout.register(IntItem, SliderWidget)
+DataSetEditLayout.register(FloatItem, SliderWidget)
 DataSetEditLayout.register(BoolItem, CheckBoxWidget)
 DataSetEditLayout.register(DateItem, DateWidget)
 DataSetEditLayout.register(DateTimeItem, DateTimeWidget)
@@ -335,7 +344,8 @@ class DataSetShowWidget(AbstractDataSetWidget):
     def __init__(self, item, parent_layout):
         AbstractDataSetWidget.__init__(self, item, parent_layout)
         self.group = QLabel()
-        self.group.setWordWrap(True)
+        wordwrap = item.get_prop_value("display", "wordwrap", False)
+        self.group.setWordWrap(wordwrap)
         self.group.setToolTip(item.get_help())
         self.group.setStyleSheet( LABEL_CSS )
         self.group.setTextInteractionFlags(Qt.TextSelectableByMouse)
@@ -426,14 +436,14 @@ DataSetShowLayout.register(FloatArrayItem, DataSetShowWidget)
 
 class DataSetShowGroupBox(QGroupBox):
     """Group box widget showing a read-only DataSet"""
-    def __init__(self, label, klass, **kwargs):
+    def __init__(self, label, klass, wordwrap=False, **kwargs):
         QGroupBox.__init__(self, label)
         self.klass = klass
         self.dataset = klass(**kwargs)
         self.layout = QVBoxLayout()
         if self.dataset.get_comment():
             label = QLabel(self.dataset.get_comment())
-            label.setWordWrap(True)
+            label.setWordWrap(wordwrap)
             self.layout.addWidget(label)
         self.grid_layout = QGridLayout()
         self.layout.addLayout(self.grid_layout)
@@ -447,7 +457,9 @@ class DataSetShowGroupBox(QGroupBox):
     def get(self):
         """Update group box contents from data item values"""
         for widget in self.edit.widgets:
+            widget.build_mode = True
             widget.get()
+            widget.build_mode = False
 
 
 class DataSetEditGroupBox(DataSetShowGroupBox):
@@ -460,8 +472,9 @@ class DataSetEditGroupBox(DataSetShowGroupBox):
     button_icon: QIcon object or string (default "apply.png")
     """
     def __init__(self, label, klass, button_text=None, button_icon=None,
-                 show_button=True, **kwargs):
-        DataSetShowGroupBox.__init__(self, label, klass, **kwargs)
+                 show_button=True, wordwrap=False, **kwargs):
+        DataSetShowGroupBox.__init__(self, label, klass, wordwrap=wordwrap,
+                                     **kwargs)
         if show_button:
             if button_text is None:
                 button_text = _("Apply")
@@ -492,4 +505,3 @@ class DataSetEditGroupBox(DataSetShowGroupBox):
         if not app_name:
             app_name = unicode(self.title())
         return "%s - %s" % ( app_name, item.label() )
-
