@@ -20,6 +20,7 @@ import time
 import subprocess
 import os
 import os.path as osp
+import locale
 
 
 #==============================================================================
@@ -90,6 +91,13 @@ def trace(fct):
 #==============================================================================
 # Strings
 #==============================================================================
+def fs_to_unicode(string):
+    """Convert string from file system charset to unicode"""
+    fs_encoding = sys.getfilesystemencoding()
+    if fs_encoding is None:
+        fs_encoding = locale.getpreferredencoding()
+    return string.decode(fs_encoding)
+
 def utf8_to_unicode(string):
     """Convert UTF-8 string to Unicode"""
     if not isinstance(string, basestring):
@@ -97,9 +105,10 @@ def utf8_to_unicode(string):
     if not isinstance(string, unicode):
         try:
             string = unicode(string, "utf-8")
-        except UnicodeDecodeError, error:
-            message = "String %r is not UTF-8 encoded"
-            raise UnicodeDecodeError(message % string, *error.args[1:])
+        except UnicodeDecodeError:
+            # This is border line... but we admit here string which has been 
+            # erroneously encoded in file system charset instead of UTF-8
+            string = fs_to_unicode(string)
     return string
 
 
@@ -122,8 +131,9 @@ def update_dataset(dest, source, visible_only=False):
     """
     Update `dest` dataset items from `source` dataset
     
-    dest should inherit from DataSet, whereas source can be any
-    Python object containing matching attribute names.
+    dest should inherit from DataSet, whereas source can be:
+        * any Python object containing matching attribute names
+        * or a dictionary with matching key names
     
     For each DataSet item, the function will try to get the attribute
     of the same name from the source. 
@@ -131,7 +141,8 @@ def update_dataset(dest, source, visible_only=False):
     visible_only: if True, update only visible items
     """
     for item in dest._items:
-        if hasattr(source, item._name):
+        key = item._name
+        if hasattr(source, key):
             try:
                 hide = item.get_prop_value("display", source, "hide", False)
             except AttributeError:
@@ -139,7 +150,9 @@ def update_dataset(dest, source, visible_only=False):
                 hide = False
             if visible_only and hide:
                 continue
-            setattr(dest, item._name, getattr(source, item._name))
+            setattr(dest, key, getattr(source, key))
+        elif isinstance(source, dict) and key in source:
+            setattr(dest, key, source[key])
 
 def restore_dataset(source, dest):
     """
@@ -147,10 +160,20 @@ def restore_dataset(source, dest):
     
     This function is almost the same as update_dataset but requires
     the source to be a DataSet instead of the destination.
+
+    Symetrically from update_dataset, `dest` may also be a dictionary.
     """
     for item in source._items:
-        if hasattr(dest, item._name):
-            setattr(dest, item._name, getattr(source, item._name))
+        key = item._name
+        value = getattr(source, key)
+        if hasattr(dest, key):
+            try:
+                setattr(dest, key, value)
+            except AttributeError:
+                # This attribute is a property, skipping this iteration
+                continue
+        elif isinstance(dest, dict):
+            dest[key] = value
 
 
 #==============================================================================
@@ -286,3 +309,32 @@ def is_module_available(module_name):
         return True
     except ImportError:
         return False
+
+
+#==============================================================================
+# Utilities for setup.py scripts
+#==============================================================================
+
+def get_package_data(name, extlist, exclude_dirs=[]):
+    """
+    Return data files for package *name* with extensions in *extlist*
+    (search recursively in package directories)
+    """
+    assert isinstance(extlist, (list, tuple))
+    flist = []
+    # Workaround to replace os.path.relpath (not available until Python 2.6):
+    offset = len(name)+len(os.pathsep)
+    for dirpath, _dirnames, filenames in os.walk(name):
+        if dirpath not in exclude_dirs:
+            for fname in filenames:
+                if osp.splitext(fname)[1].lower() in extlist:
+                    flist.append(osp.join(dirpath, fname)[offset:])
+    return flist
+
+def get_subpackages(name):
+    """Return subpackages of package *name*"""
+    splist = []
+    for dirpath, _dirnames, _filenames in os.walk(name):
+        if osp.isfile(osp.join(dirpath, '__init__.py')):
+            splist.append(".".join(dirpath.split(os.sep)))
+    return splist
